@@ -26,10 +26,12 @@ Rcpp::List bsf(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
       
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
@@ -50,10 +52,12 @@ Rcpp::List bsf(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
       
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
@@ -74,10 +78,12 @@ Rcpp::List bsf(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
       
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
@@ -101,10 +107,12 @@ Rcpp::List bsf(const Rcpp::List model_,
     unsigned int m = model.m;
     unsigned n = model.n;
     
-    arma::cube alpha(m, n + 1, nsim);
-    arma::mat weights(nsim, n + 1);
-    arma::umat indices(nsim, n);
+    arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+    arma::mat weights(nsim, n + 1, arma::fill::zeros);
+    arma::umat indices(nsim, n, arma::fill::zeros);
     double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+    if (!std::isfinite(loglik)) 
+      Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
     
     arma::mat at(m, n + 1);
     arma::mat att(m, n);
@@ -125,10 +133,12 @@ Rcpp::List bsf(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
       
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
@@ -149,10 +159,13 @@ Rcpp::List bsf(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
       arma::cube Pt(m, m, n + 1);
@@ -173,10 +186,13 @@ Rcpp::List bsf(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
       arma::cube Pt(m, m, n + 1);
@@ -197,10 +213,13 @@ Rcpp::List bsf(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat at(m, n + 1);
       arma::mat att(m, n);
       arma::cube Pt(m, m, n + 1);
@@ -237,16 +256,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
       
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      summary(alpha, alphahat, Vt);
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -259,15 +280,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      summary(alpha, alphahat, Vt);
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -281,15 +305,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      summary(alpha, alphahat, Vt);
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -306,16 +333,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
     unsigned int m = model.m;
     unsigned n = model.n;
     
-    arma::cube alpha(m, n + 1, nsim);
-    arma::mat weights(nsim, n + 1);
-    arma::umat indices(nsim, n);
+    arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+    arma::mat weights(nsim, n + 1, arma::fill::zeros);
+    arma::umat indices(nsim, n, arma::fill::zeros);
     double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+    if (!std::isfinite(loglik)) 
+      Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
     
     arma::mat alphahat(model.m, model.n + 1);
     arma::cube Vt(model.m, model.m, model.n + 1);
     
     filter_smoother(alpha, indices);
-    weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+    summary(alpha, alphahat, Vt);
     
     arma::inplace_trans(alphahat);
     return Rcpp::List::create(
@@ -328,16 +357,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
       
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      summary(alpha, alphahat, Vt);
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -350,15 +381,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      summary(alpha, alphahat, Vt);
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -372,15 +406,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      summary(alpha, alphahat, Vt);
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -394,15 +431,18 @@ Rcpp::List bsf_smoother(const Rcpp::List model_,
       unsigned int m = model.m;
       unsigned n = model.n;
       
-      arma::cube alpha(m, n + 1, nsim);
-      arma::mat weights(nsim, n + 1);
-      arma::umat indices(nsim, n);
+      arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+      arma::mat weights(nsim, n + 1, arma::fill::zeros);
+      arma::umat indices(nsim, n, arma::fill::zeros);
       double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+      if (!std::isfinite(loglik)) 
+        Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+      
       arma::mat alphahat(model.m, model.n + 1);
       arma::cube Vt(model.m, model.m, model.n + 1);
       
       filter_smoother(alpha, indices);
-      weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+      summary(alpha, alphahat, Vt);
       
       arma::inplace_trans(alphahat);
       return Rcpp::List::create(
@@ -444,11 +484,13 @@ Rcpp::List bsf_nlg(const arma::mat& y, SEXP Z, SEXP H,
   unsigned int m = model.m;
   unsigned n = model.n;
   
-  arma::cube alpha(m, n + 1, nsim);
-  arma::mat weights(nsim, n + 1);
-  arma::umat indices(nsim, n);
+  arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+  arma::mat weights(nsim, n + 1, arma::fill::zeros);
+  arma::umat indices(nsim, n, arma::fill::zeros);
   
   double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+  if (!std::isfinite(loglik)) 
+    Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
   
   arma::mat at(m, n + 1);
   arma::mat att(m, n);
@@ -491,15 +533,17 @@ Rcpp::List bsf_smoother_nlg(const arma::mat& y, SEXP Z, SEXP H,
   unsigned int m = model.m;
   unsigned n = model.n;
   
-  arma::cube alpha(m, n + 1, nsim);
-  arma::mat weights(nsim, n + 1);
-  arma::umat indices(nsim, n);
+  arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+  arma::mat weights(nsim, n + 1, arma::fill::zeros);
+  arma::umat indices(nsim, n, arma::fill::zeros);
   double loglik = model.bsf_filter(nsim, alpha, weights, indices);
+  if (!std::isfinite(loglik)) 
+    Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
   
   arma::mat alphahat(model.m, model.n + 1);
   arma::cube Vt(model.m, model.m, model.n + 1);
   filter_smoother(alpha, indices);
-  weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+  summary(alpha, alphahat, Vt);
   arma::inplace_trans(alphahat);
   
   return Rcpp::List::create(
