@@ -30,11 +30,13 @@ Rcpp::List ekpf(const arma::mat& y, SEXP Z, SEXP H,
   unsigned int m = model.m;
   unsigned n = model.n;
 
-  arma::cube alpha(m, n+1, nsim);
-  arma::mat weights(nsim, n+1);
-  arma::umat indices(nsim, n);
+  arma::cube alpha(m, n+1, nsim, arma::fill::zeros);
+  arma::mat weights(nsim, n+1, arma::fill::zeros);
+  arma::umat indices(nsim, n, arma::fill::zeros);
   double loglik = model.ekf_filter(nsim, alpha, weights, indices);
-
+  if (!std::isfinite(loglik)) 
+    Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+  
 
   arma::mat at(m, n + 1);
   arma::mat att(m, n + 1);
@@ -76,17 +78,19 @@ Rcpp::List ekpf_smoother(const arma::mat& y, SEXP Z, SEXP H,
   unsigned int m = model.m;
   unsigned n = model.n;
 
-  arma::cube alpha(m, n + 1, nsim);
-  arma::mat weights(nsim, n + 1);
-  arma::umat indices(nsim, n);
+  arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+  arma::mat weights(nsim, n + 1, arma::fill::zeros);
+  arma::umat indices(nsim, n, arma::fill::zeros);
   double loglik = model.ekf_filter(nsim, alpha, weights, indices);
-
+  if (!std::isfinite(loglik)) 
+    Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+  
 
   arma::mat alphahat(model.m, model.n + 1);
   arma::cube Vt(model.m, model.m, model.n + 1);
 
   filter_smoother(alpha, indices);
-  weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+  summary(alpha, alphahat, Vt);
 
   arma::inplace_trans(alphahat);
 
