@@ -20,25 +20,25 @@ arma::cube gaussian_psi_smoother(const Rcpp::List model_,
   switch (model_type) {
   case 0: {
   ssm_mlg model(model_, seed);
-  arma::cube alpha(model.m, model.n + 1, nsim);
+  arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
   model.psi_filter(nsim, alpha);
   return alpha;
 } break;
   case 1: {
   ssm_ulg model(model_, seed);
-  arma::cube alpha(model.m, model.n + 1, nsim);
+  arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
   model.psi_filter(nsim, alpha);
   return alpha;
 } break;
   case 2: {
     bsm_lg model(model_, seed);
-    arma::cube alpha(model.m, model.n + 1, nsim);
+    arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
     model.psi_filter(nsim, alpha);
     return alpha;
   } break;
   case 3: {
     ar1_lg model(model_, seed);
-    arma::cube alpha(model.m, model.n + 1, nsim);
+    arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
     model.psi_filter(nsim, alpha);
     return alpha;
   } break;
@@ -56,16 +56,19 @@ Rcpp::List psi_smoother(const Rcpp::List model_,
   switch (model_type) {
   case 0: {
   ssm_mng model(model_, seed);
-  arma::cube alpha(model.m, model.n + 1, nsim);
-  arma::mat weights(nsim, model.n + 1);
-  arma::umat indices(nsim, model.n);
+  arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
+  arma::mat weights(nsim, model.n + 1, arma::fill::zeros);
+  arma::umat indices(nsim, model.n, arma::fill::zeros);
   
   double loglik = model.psi_filter(nsim, alpha, weights, indices);
+  if (!std::isfinite(loglik)) 
+    Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+  
   arma::mat alphahat(model.m, model.n + 1);
   arma::cube Vt(model.m, model.m, model.n + 1);
   
   filter_smoother(alpha, indices);
-  weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+  summary(alpha, alphahat, Vt); // weights are uniform due to extra time point
   
   arma::inplace_trans(alphahat);
   return Rcpp::List::create(
@@ -77,16 +80,19 @@ Rcpp::List psi_smoother(const Rcpp::List model_,
   case 1: {
   ssm_ung model(model_, seed);
   
-  arma::cube alpha(model.m, model.n + 1, nsim);
-  arma::mat weights(nsim, model.n + 1);
-  arma::umat indices(nsim, model.n);
+  arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
+  arma::mat weights(nsim, model.n + 1, arma::fill::zeros);
+  arma::umat indices(nsim, model.n, arma::fill::zeros);
   
   double loglik = model.psi_filter(nsim, alpha, weights, indices);
+  if (!std::isfinite(loglik)) 
+    Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+  
   arma::mat alphahat(model.m, model.n + 1);
   arma::cube Vt(model.m, model.m, model.n + 1);
   
   filter_smoother(alpha, indices);
-  weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+  summary(alpha, alphahat, Vt); // weights are uniform due to extra time point
   
   arma::inplace_trans(alphahat);
   return Rcpp::List::create(
@@ -96,17 +102,19 @@ Rcpp::List psi_smoother(const Rcpp::List model_,
 } break;
   case 2: {
     bsm_ng model(model_, seed);
-    arma::cube alpha(model.m, model.n + 1, nsim);
-    arma::mat weights(nsim, model.n + 1);
-    arma::umat indices(nsim, model.n);
+    arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
+    arma::mat weights(nsim, model.n + 1, arma::fill::zeros);
+    arma::umat indices(nsim, model.n, arma::fill::zeros);
     
     double loglik = model.psi_filter(nsim, alpha, weights, indices);
+    if (!std::isfinite(loglik)) 
+      Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
     
     arma::mat alphahat(model.m, model.n + 1);
     arma::cube Vt(model.m, model.m, model.n + 1);
     
     filter_smoother(alpha, indices);
-    weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+    summary(alpha, alphahat, Vt); // weights are uniform due to extra time point
     
     arma::inplace_trans(alphahat);
     return Rcpp::List::create(
@@ -116,17 +124,19 @@ Rcpp::List psi_smoother(const Rcpp::List model_,
   } break;
   case 3: {
     svm model(model_, seed);
-    arma::cube alpha(model.m, model.n + 1, nsim);
-    arma::mat weights(nsim, model.n + 1);
-    arma::umat indices(nsim, model.n);
+    arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
+    arma::mat weights(nsim, model.n + 1, arma::fill::zeros);
+    arma::umat indices(nsim, model.n, arma::fill::zeros);
     
     double loglik = model.psi_filter(nsim, alpha, weights, indices);
+    if (!std::isfinite(loglik)) 
+      Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
     
     arma::mat alphahat(model.m, model.n + 1);
     arma::cube Vt(model.m, model.m, model.n + 1);
     
     filter_smoother(alpha, indices);
-    weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+    summary(alpha, alphahat, Vt); // weights are uniform due to extra time point
     
     arma::inplace_trans(alphahat);
     return Rcpp::List::create(
@@ -136,17 +146,19 @@ Rcpp::List psi_smoother(const Rcpp::List model_,
   } break;
   case 4: {
     ar1_ng model(model_, seed);
-    arma::cube alpha(model.m, model.n + 1, nsim);
-    arma::mat weights(nsim, model.n + 1);
-    arma::umat indices(nsim, model.n);
+    arma::cube alpha(model.m, model.n + 1, nsim, arma::fill::zeros);
+    arma::mat weights(nsim, model.n + 1, arma::fill::zeros);
+    arma::umat indices(nsim, model.n, arma::fill::zeros);
     
     double loglik = model.psi_filter(nsim, alpha, weights, indices);
+    if (!std::isfinite(loglik)) 
+      Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
     
     arma::mat alphahat(model.m, model.n + 1);
     arma::cube Vt(model.m, model.m, model.n + 1);
     
     filter_smoother(alpha, indices);
-    weighted_summary(alpha, alphahat, Vt, weights.col(model.n));
+    summary(alpha, alphahat, Vt); // weights are uniform due to extra time point
     
     arma::inplace_trans(alphahat);
     return Rcpp::List::create(
@@ -190,16 +202,18 @@ Rcpp::List psi_smoother_nlg(const arma::mat& y, SEXP Z, SEXP H,
   if(!arma::is_finite(model.mode_estimate)) {
     Rcpp::warning("Approximation did not converge. ");
   }
-  arma::cube alpha(m, n + 1, nsim);
-  arma::mat weights(nsim, n + 1);
-  arma::umat indices(nsim, n);
+  arma::cube alpha(m, n + 1, nsim, arma::fill::zeros);
+  arma::mat weights(nsim, n + 1, arma::fill::zeros);
+  arma::umat indices(nsim, n, arma::fill::zeros);
   double loglik = model.psi_filter(nsim, alpha, weights, indices);
-
+  if (!std::isfinite(loglik)) 
+    Rcpp::warning("Particle filtering stopped prematurely due to nonfinite log-likelihood.");
+  
   arma::mat alphahat(model.m, model.n + 1);
   arma::cube Vt(model.m, model.m, model.n + 1);
 
   filter_smoother(alpha, indices);
-  weighted_summary(alpha, alphahat, Vt, weights.col(n));
+  summary(alpha, alphahat, Vt); // weights are uniform due to extra time point
   arma::inplace_trans(alphahat);
   return Rcpp::List::create(
     Rcpp::Named("alphahat") = alphahat, Rcpp::Named("Vt") = Vt,
