@@ -1,9 +1,8 @@
 #ifndef BSSM_H
 #define BSSM_H
 
-//#define ARMA_NO_DEBUG
+#define ARMA_NO_DEBUG
 #include <RcppArmadillo.h>
-// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::plugins(cpp11)]]
 
 #endif
